<?php
/**
 * 导航数据迁移
 * User: lzs
 * Date: 2020/11/26
 * Time: 10:48
 * Brand: ePower
 */

namespace App\Services\Migration;


use App\Models\Attachment;
use App\Models\Navigation;
use App\Models\RouteSeo;
use App\Services\MigrationService;
use Illuminate\Database\ConnectionInterface;
use Illuminate\Support\Str;

class Navigations
{
    public static function handle(ConnectionInterface $connection, $page = 1)
    {
        // 导航数据迁移
        $data = self::navigations($connection, $page);

        // 生成 SQL 语句
        MigrationService::insert('navigations', $data['navigations'], $page);

        return [
            'is_finished' => $data['is_finished'],
        ];
    }

    /**
     * 获取导航数据
     * @param ConnectionInterface $connection
     * @param $page
     * @return array
     * @brand ePower
     * @author lzs
     */
    private static function navigations(ConnectionInterface $connection, $page)
    {
        $baseQuery = $connection->table('navigation')->where('position', 'top');

        $count = $baseQuery->count();

        $offset = ($page - 1) * MigrationService::PAGE_LIMIT;
        $navs = $baseQuery->select(['id', 'name', 'url', 'target', 'orderby', 'status', 'title', 'keywords', 'description', 'tag', 'default_index', 'is_label', 'label', 'isdetails', 'details', 'pic', 'pic_url', 'helpCid', 'position'])
            ->offset($offset)
            ->limit(MigrationService::PAGE_LIMIT)
            ->get();

        $attachmentPaths = [];
        foreach ($navs as $nav){
            if ($nav->pic){
                $attachmentPaths[] = $nav->pic;
            }
        }
        $attachments = Attachment::query()->whereIn('path', $attachmentPaths)->pluck('id', 'path')->all();

        // 处理数据
        $data = [];
        $time = date('Y-m-d H:i:s');
        $targets = [
            1 => '_self',
            2 => '_blank',
        ];


        foreach ($navs  as $nav){
            $link = $nav->url;
            if ($nav->url == '/home'){
                $link = '/';
            }
            $target = isset($targets[$nav->target]) ? $targets[$nav->target] : '_self';
            $subNav = self::parseDetails($nav->details);
            $data[] = [
                'order' => $nav->orderby,
                'type' => Navigation::TYPE_TOP,
                'identity' => $nav->tag,
                'name' => $nav->name,
                'tag' => $nav->is_label ? $nav->label : '',
                'link' => $link ?: '',
                'target' => $target,
                'help_category_id' => $nav->helpCid ?: '',
                'banner_id' => isset($attachments[$nav->pic]) ? $attachments[$nav->pic] : 0,
                'banner_link' => $nav->pic_url ?: '',
                'is_home' => $nav->default_index ? Navigation::IS_HOME_YES : Navigation::IS_HOME_NO,
                'sub_nav' => json_encode($subNav),
                'sub_nav_enable' => $nav->isdetails ? Navigation::SUB_NAV_ENABLE_YES : Navigation::SUB_NAV_ENABLE_NO,
                'enable' => $nav->status ? Navigation::ENABLE_YES : Navigation::ENABLE_NO,
                'created_at' => $time,
                'updated_at' => $time,
            ];

            // SEO 数据迁移
            if ($link){
                RouteSeo::updateOrCreate(['route' => $link], ['title' => $nav->title, 'keywords' => $nav->keywords, 'description' => $nav->description, 'enable' => RouteSeo::ENABLE_YES]);
            }
        }

        $isFinished = false;
        if ($page * MigrationService::PAGE_LIMIT >= $count){
            $isFinished = true;
        }

        return [
            'is_finished' => $isFinished,
            'navigations' => $data
        ];
    }

    /**
     * 解析子导航
     * @param string $details
     * @return string|array
     * @brand ePower
     * @author lzs
     */
    private static function parseDetails($details)
    {
        if (!$details){
            return '';
        }

        $details = json_decode($details, true);
        if (!$details){
            return '';
        }
        $subNav = [];
        $i = 1000;
        $tags = [
            1 => 'new',
            2 => 'hot'
        ];
        $targets = [
            0 => '_blank',
            1 => '_self',
        ];
        foreach ($details as $key => $detail){
            $tempNav = [
                'id' => $i,
                'name' => $key,
            ];
            $children = [];
            if ($detail){
                foreach ($detail as $child){
                    $url = self::transformUrl(htmlspecialchars_decode($child['url']));
                    if ($url === false){
                        continue;
                    }
                    $i++;
                    $children[] = [
                        'id' => $i,
                        'name' => $child['name'],
                        'target' => isset($targets[$child['open']]) ? $targets[$child['open']] : '_self',
                        'tag' => isset($tags[$child['tag']]) ? $tags[$child['tag']] : '',
                        'order' => 0,
                        'url' => $url,
                    ];
                }
            }
            $tempNav['children'] = $children;
            $subNav[] = $tempNav;
            $i++;
        }

        return $subNav;
    }

    /**
     * url 转换
     * @param $url
     * @return string
     * @brand ePower
     * @author lzs
     */
    private static function transformUrl($url)
    {
        $info = parse_url($url);
        if (!isset($info['path'])){
            return $url;
        }

        $invalidUrls = [
            '/user/dbconsole',
            '/cloudhelper',
            '/mina',
            '/domain/single/govcn',
        ];
        if (in_array(strtolower($info['path']), $invalidUrls)){
            return false;
        }
        if ($url == '/server?id=9'){
            return false;
        }

        if ($url == '/help/price?item=1'){
            return '/domain/batchregistered';
        }
        if (Str::startsWith($info['path'], '/domain/single/')){
            $domainType = Str::after($info['path'], '/domain/single/');
            return '/domain/detail?type='.$domainType;
        }
        if (Str::startsWith($info['path'], '/xcx/buy/')){
            $id = Str::after($info['path'], '/xcx/buy/');
            return '/xcx/buy?id='.$id;
        }
        if (Str::startsWith($info['path'], '/ssl/detail/')){
            $sslType = Str::after($info['path'], '/ssl/detail/');
            return '/ssl/detail?type='.$sslType;
        }
        if (Str::startsWith($info['path'], '/notices/content/tnotice/')){
            $id = Str::after($info['path'], '/notices/content/tnotice/');
            return '/notices/detail?id='.$id;
        }
        if (Str::startsWith($info['path'], '/trademark/jiaoyidetails/')){
            $id = Str::after($info['path'], '/trademark/jiaoyidetails/');
            return '/trademark/jiaoyidetails?tm_id='.$id;
        }

        switch (rtrim(strtolower($info['path']), '/')){
            case '/server':
                if (!isset($info['query'])) return $url;
                $params = explode('&', $info['query']);
                $tempParams = [];
                foreach ($params as $param){
                    list($key, $value) = explode('=', $param);
                    if (strtolower($key) != 'id'){
                        continue;
                    }
                    $tempParams['product_id'] = $value;
                }
                return '/server?'.http_build_query($tempParams);
                break;
            case '/host/buy':
            case '/host/buy.html':
                if (!isset($info['query'])) return $url;
                $params = explode('&', $info['query']);
                $tempParams = [];
                $queryMap = [
                    'pid' => 'product_id',
                    'lineid' => 'line_id',
                ];
                foreach ($params as $param){
                    list($key, $value) = explode('=', $param);
                    $key = strtolower($key);
                    if (isset($queryMap[$key])){
                        $tempParams[$queryMap[$key]] = $value;
                    }
                }
                return '/host/buy?'.http_build_query($tempParams);
                break;
            case '/trademark/services':
                if (!isset($info['query'])) return $url;
                $params = explode('&', $info['query']);
                $tempParams = [];
                foreach ($params as $param){
                    list($key, $value) = explode('=', $param);
                    if (strtolower($key) != 'remindinfo'){
                        continue;
                    }
                    $tempParams['type'] = $value;
                }
                return '/trademark/services/detail?'.http_build_query($tempParams);
                break;
            case '/ssl/order':
                if (!isset($info['query'])) return $url;
                $classes = [
                    1 => 'DV',
                    3 => 'OV',
                    4 => 'EV',
                ];
                $params = explode('&', $info['query']);
                $tempParams = [];
                $queryMap = [
                    'certype' => 'type',
                    'product_id' => 'id',
                ];
                foreach ($params as $param){
                    list($key, $value) = explode('=', $param);
                    $key = strtolower($key);
                    if (!in_array($key, array_keys($queryMap))){
                        continue;
                    }
                    if ($key == 'certype'){
                        $tempParams[$queryMap[$key]] = isset($classes[$value]) ? $classes[$value] : '';
                    }
                    if ($key == 'product_id'){
                        $value = $value == 11 ? 16 : $value;
                        $tempParams[$queryMap[$key]] = $value;
                    }
                }
                return '/ssl/order?'.http_build_query($tempParams);
                break;
            case '/trademark/cls':
                return '/trademark/classes';
                break;
            case '/trademark/danbao':
                return '/trademark/form?type=danbao';
                break;
            case '/user/servermanage':
                return '/user/server';
                break;
            case '/user/hostmanage':
                return '/user/host';
                break;
            case '/user/miniprogram/index':
                return '/user/xcx/index';
                break;
            case '/domain/serch':
                return '/domain/whois';
                break;
            case '/ymjy':
                return '/domain/jiaoyi';
                break;
            case '/ymjy/dt/publish':
                return '/user/ymjy/publish';
                break;
            default:
                return $url;
                break;
        }

    }

}
