<?php
/**
 * 财务明细数据迁移
 * User: lzs
 * Date: 2020/10/21
 * Time: 11:01
 * Brand: ePower
 */

namespace App\Services\Migration;


use App\Models\UserBill;
use App\Services\MigrationService;
use Illuminate\Database\ConnectionInterface;

class UserBills
{
    public static function handle(ConnectionInterface $connection, $page = 1)
    {
        // 财务明细数据迁移
        $data = self::bills($connection, $page);

        // 生成 SQL 语句
        MigrationService::insert('user_bills', $data['bills'], $page);

        return [
            'is_finished' => $data['is_finished'],
        ];
    }

    /**
     * 获取财务明细数据
     * @param ConnectionInterface $connection
     * @param $page
     * @return array
     * @brand ePower
     * @author lzs
     */
    private static function bills(ConnectionInterface $connection, $page)
    {
        $baseQuery = $connection->table('users_money');

        $count = $baseQuery->count();

        $offset = ($page - 1) * MigrationService::PAGE_LIMIT;
        $bills = $baseQuery->select(['id', 'userID', 'type', 'serviceType', 'serviceID', 'money', 'balance', 'description', 'description_inner', 'dateTime'])
            ->offset($offset)
            ->limit(MigrationService::PAGE_LIMIT)
            ->get();

        // 处理数据
        $data = [];

        foreach ($bills  as $bill){
            $typeInfo = self::typeInfo($bill, $connection);
            $type = $typeInfo['type'];
            $serviceType = $typeInfo['service_type'];
            $objId = $typeInfo['obj_id'];

            $data[] = [
                'id' => $bill->id,
                'user_id' => $bill->userID,
                'type' => $type,
                'service_type' => $serviceType,
                'object_id' => $objId,
                'money' => bcmul($bill->money, 100, 0),
                'balance' => bcmul($bill->balance, 100, 0),
                'description' => $bill->description ?? '',
                'description_inner' => $bill->description_inner ?? '',
                'created_at' => $bill->dateTime,
                'updated_at' => $bill->dateTime,
            ];
        }

        $isFinished = false;
        if ($page * MigrationService::PAGE_LIMIT >= $count){
            $isFinished = true;
        }

        return [
            'is_finished' => $isFinished,
            'bills' => $data
        ];
    }

    /**
     * 获取财务账单相关信息
     * @param $bill
     * @param ConnectionInterface $connection
     * @return array
     * @brand ePower
     * @author lzs
     */
    private static function typeInfo($bill, ConnectionInterface $connection)
    {
        // 充值入款 recharge
        if ($bill->type == 'recharge' && $bill->serviceType == 'admin'){
            return ['type' => UserBill::TYPE_RECHARGE, 'service_type' => UserBill::SERVICE_TYPE_ADMIN_RECEIPT, 'obj_id' => 0];
        }
        if ($bill->type == 'recharge' && $bill->serviceType == 'recharge'){
            return ['type' => UserBill::TYPE_RECHARGE, 'service_type' => UserBill::SERVICE_TYPE_USER_RECHARGE, 'obj_id' => intval($bill->serviceID)];
        }

        // 退款 refund
        if ($bill->type == 'refund' && $bill->serviceType == 'domain_transfer'){
            return ['type' => UserBill::TYPE_REFUND, 'service_type' => UserBill::SERVICE_TYPE_ORDER_DOMAIN_TRANSFER_IN, 'obj_id' => 0];
        }
        if ($bill->type == 'refund'){
            return ['type' => UserBill::TYPE_REFUND, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
        }

        // 赠送 giveaway
        if ($bill->type == 'giveaway'){
            return ['type' => UserBill::TYPE_OTHERS, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
        }

        // 提现 withdrawal
        if ($bill->type == 'withdrawal'){
            return ['type' => UserBill::TYPE_WITHDRAW, 'service_type' => UserBill::SERVICE_TYPE_WITHDRAW, 'obj_id' => intval($bill->serviceID)];
        }

        // 交易 trade
        if ($bill->type == 'trade'){
            return ['type' => UserBill::TYPE_OTHERS, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
        }

        // 手续费 fee
        if ($bill->type == 'fee' && $bill->serviceType == 'withdrawal'){
            return ['type' => UserBill::TYPE_FEE, 'service_type' => UserBill::SERVICE_TYPE_WITHDRAW_SERVICE, 'obj_id' => intval($bill->serviceID)];
        }
        if ($bill->type == 'fee' && $bill->serviceType == 'invoice'){
            $bill->serviceType = (mb_strpos($bill->description, '手续') !== false) ? UserBill::SERVICE_TYPE_INVOICE_SERVICE : UserBill::SERVICE_TYPE_INVOICE_EXPRESS;
            return ['type' => UserBill::TYPE_FEE, 'service_type' => $bill->serviceType, 'obj_id' => intval($bill->serviceID)];
        }
        if ($bill->type == 'consume' && $bill->serviceType == 'fawenjifu'){
            return ['type' => UserBill::TYPE_FEE, 'service_type' => UserBill::SERVICE_TYPE_FILE_EXPRESS, 'obj_id' => intval($bill->serviceID)];
        }

        // 佣金 commission
        if ($bill->type == 'commission' && $bill->serviceType == 'system'){
            return ['type' => UserBill::TYPE_COMMISSION, 'service_type' => UserBill::SERVICE_TYPE_COMMISSION_RECEIPT, 'obj_id' => 0];
        }

        // 订单 order
        if ($bill->type == 'order'){
            return ['type' => UserBill::TYPE_OTHERS, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
        }

        // 扣款
        if ($bill->type == 'consume' && $bill->serviceType == 'admin'){
            return ['type' => UserBill::TYPE_EXPENSE, 'service_type' => UserBill::SERVICE_TYPE_ADMIN_EXPENSE, 'obj_id' => 0];
        }

        // 消费 consume
        if ($bill->type == 'consume' && $bill->serviceType == 'ipr'){
            $iprOrder = $connection->table('ipr_orders')->where('id', $bill->serviceID)->first();
            if (!$iprOrder){
                return ['type' => UserBill::TYPE_ORDER, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
            }
            if (in_array($iprOrder->product_type, ['faming', 'shiyong', 'waiguan', 'zhuanlibohui'])){
                return ['type' => UserBill::TYPE_ORDER, 'service_type' => UserBill::SERVICE_TYPE_ORDER_TM_PATENT, 'obj_id' => 0];
            }
            if (in_array($iprOrder->product_type, ['meishu', 'wenzi', 'ruanjian', 'qita'])){
                return ['type' => UserBill::TYPE_ORDER, 'service_type' => UserBill::SERVICE_TYPE_ORDER_TM_COPYRIGHT, 'obj_id' => 0];
            }
        }

        if ($bill->type == 'consume'){
            $serviceTypeMaps = [
                'order' => UserBill::SERVICE_TYPE_OTHERS,
                'icpcode' => UserBill::SERVICE_TYPE_ORDER_GW_ICP_CODE_BUY,
                'serverrefund' => UserBill::SERVICE_TYPE_OTHERS,
                'serverpush' => UserBill::SERVICE_TYPE_OTHERS,
                'recharge' => UserBill::SERVICE_TYPE_OTHERS,
                'invoice' => UserBill::SERVICE_TYPE_OTHERS,
                'contract' => UserBill::SERVICE_TYPE_OTHERS,
                'withdrawal' => UserBill::SERVICE_TYPE_OTHERS,
                'system' => UserBill::SERVICE_TYPE_OTHERS,
                'idc' => UserBill::SERVICE_TYPE_OTHERS,
                'host' => UserBill::SERVICE_TYPE_OTHERS,
                'others' => UserBill::SERVICE_TYPE_OTHERS,
                'domain_reg' => UserBill::SERVICE_TYPE_ORDER_DOMAIN_REG,
                'domain_renew' => UserBill::SERVICE_TYPE_ORDER_DOMAIN_RENEW,
                'domain_transfer' => UserBill::SERVICE_TYPE_ORDER_DOMAIN_TRANSFER_IN,
                'domain_restore' => UserBill::SERVICE_TYPE_OTHERS,
                'host_buy' => UserBill::SERVICE_TYPE_OTHERS,
                'host_renew' => UserBill::SERVICE_TYPE_OTHERS,
                'sql_buy' => UserBill::SERVICE_TYPE_OTHERS,
                'sql_renew' => UserBill::SERVICE_TYPE_OTHERS,
                'mail_buy' => UserBill::SERVICE_TYPE_OTHERS,
                'mail_renew' => UserBill::SERVICE_TYPE_OTHERS,
                'tmbuy' => UserBill::SERVICE_TYPE_ORDER_TM_REG,
                'tmsbuy' => UserBill::SERVICE_TYPE_ORDER_TM_SERVICE,
                'website' => UserBill::SERVICE_TYPE_ORDER_GW_CREATE,
                'removecr' => UserBill::SERVICE_TYPE_ORDER_GW_COPYRIGHT,
                'refund' => UserBill::SERVICE_TYPE_OTHERS,
                'tmaddservice' => UserBill::SERVICE_TYPE_ORDER_TM_ADD_SERVICE,
                'tmjiaoyi_bond' => UserBill::SERVICE_TYPE_ORDER_TM_TRADE_DEPOSIT,
                'tmjiaoyi_service' => UserBill::SERVICE_TYPE_ORDER_TM_TRADE_SERVICE,
                'tmjiaoyi_end' => UserBill::SERVICE_TYPE_ORDER_TM_TRADE_TAIL,
                'domain' => UserBill::SERVICE_TYPE_ORDER_GW_DOMAIN_BUY,
                'ssl' => UserBill::SERVICE_TYPE_ORDER_SSL_BUY,
                'zz' => UserBill::SERVICE_TYPE_ORDER_VAS_BUY,
                'etpemail_reg' => UserBill::SERVICE_TYPE_ORDER_ETP_EMAIL_PURCHASE,
                'etpemail_renew' => UserBill::SERVICE_TYPE_ORDER_ETP_EMAIL_RENEW,
                'website_renew' => UserBill::SERVICE_TYPE_ORDER_GW_RENEW,
                'etpemail_expand' => UserBill::SERVICE_TYPE_ORDER_ETP_EMAIL_UPGRADE,
                'miniprogram_pic' => UserBill::SERVICE_TYPE_MINI_PROGRAM_STORAGE_BUY,
                'miniprogram_sms' => UserBill::SERVICE_TYPE_MINI_PROGRAM_SMS_BUY,
                'miniprogram_added' => UserBill::SERVICE_TYPE_MINI_PROGRAM_FUNCTION_OPEN,
                'miniprogram_create' => UserBill::SERVICE_TYPE_MINI_PROGRAM_CREATE,
                'miniprogram_renew' => UserBill::SERVICE_TYPE_MINI_PROGRAM_RENEW,
                'miniprogram_upgrade' => UserBill::SERVICE_TYPE_MINI_PROGRAM_UPGRADE,
                'domaintrade_buy' => UserBill::SERVICE_TYPE_DOMAIN_TRADE,
                'companyservice' => UserBill::SERVICE_TYPE_ORDER_CUSTOM_SERVICE_BUY,
                'tm_international' => UserBill::SERVICE_TYPE_ORDER_TM_INTERNATIONAL,
                'tm_notarization' => UserBill::SERVICE_TYPE_ORDER_TM_NOTARIZATION,
                'miniprogram_addedrenew' => UserBill::SERVICE_TYPE_MINI_PROGRAM_FUNCTION_RENEW,
                'website_upgrade' => UserBill::SERVICE_TYPE_ORDER_GW_UPGRADE,
                'website_agent_construction' => UserBill::SERVICE_TYPE_ORDER_GW_AGENT_CONSTRUCT,
            ];
            $serviceType = isset($serviceTypeMaps[$bill->serviceType]) ? $serviceTypeMaps[$bill->serviceType] : UserBill::SERVICE_TYPE_OTHERS;

            return ['type' => UserBill::TYPE_ORDER, 'service_type' => $serviceType, 'obj_id' => 0];
        }

        return ['type' => UserBill::TYPE_OTHERS, 'service_type' => UserBill::SERVICE_TYPE_OTHERS, 'obj_id' => 0];
    }

}
